@@ -19,7 +19,7 @@
 #ifndef vte_vte_h_included
 #define vte_vte_h_included
 
-#ident "$Id: vte.h,v 1.64 2003/03/07 00:28:21 nalin Exp $"
+#ident "$Id: vte.h,v 1.68 2003/05/29 04:50:48 nalin Exp $"
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -121,6 +121,13 @@ struct _VteCharAttributes {
 };
 typedef struct _VteCharAttributes VteCharAttributes;
 
+/* The name of the same structure in the 0.10 series, for API compatibility. */
+struct vte_char_attributes {
+	long row, column;
+	GdkColor fore, back;
+	gboolean underline:1, strikethrough:1;
+};
+
 /* The widget's type. */
 GtkType vte_terminal_get_type(void);
 GtkType vte_terminal_erase_binding_get_type(void);
@@ -217,7 +224,7 @@ void vte_terminal_im_append_menuitems(VteTerminal *terminal,
 
 /* Set or retrieve the current font. */
 void vte_terminal_set_font(VteTerminal *terminal,
-                           const PangoFontDescription *font_desc);
+			   const PangoFontDescription *font_desc);
 void vte_terminal_set_font_from_string(VteTerminal *terminal, const char *name);
 const PangoFontDescription *vte_terminal_get_font(VteTerminal *terminal);
 gboolean vte_terminal_get_using_xft(VteTerminal *terminal);
@@ -281,6 +288,8 @@ int vte_terminal_match_add(VteTerminal *terminal, const char *match);
 /* Set the cursor to be used when the pointer is over a given match. */
 void vte_terminal_match_set_cursor(VteTerminal *terminal, int tag,
 				   GdkCursor *cursor);
+void vte_terminal_match_set_cursor_type(VteTerminal *terminal,
+					int tag, GdkCursorType cursor_type);
 /* Remove a matching expression by tag. */
 void vte_terminal_match_remove(VteTerminal *terminal, int tag);
 
@@ -316,6 +325,9 @@ glong vte_terminal_get_column_count(VteTerminal *terminal);
 const char *vte_terminal_get_window_title(VteTerminal *terminal);
 const char *vte_terminal_get_icon_title(VteTerminal *terminal);
 
+/* You never saw this. */
+void _vte_terminal_accessible_ref(VteTerminal *terminal);
+
 G_END_DECLS
 
 #endif
