@@ -19,8 +19,10 @@
 /* VTE accessibility object.  Based heavily on inspection of libzvt's
  * accessibility code. */
 
-#ident "$Id: vteaccess.c,v 1.31 2003/04/15 16:40:15 nalin Exp $"
+#ident "$Id: vteaccess.c,v 1.40 2003/05/30 08:55:28 padraigo Exp $"
+
 #include "../config.h"
+
 #include <atk/atk.h>
 #include <gtk/gtk.h>
 #include <string.h>
@@ -39,6 +41,7 @@
 #endif
 
 #define VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA "VteTerminalAccessiblePrivateData"
+
 typedef struct _VteTerminalAccessiblePrivate {
 	gboolean snapshot_contents_invalid;	/* This data is stale. */
 	gboolean snapshot_caret_invalid;	/* This data is stale. */
@@ -58,6 +61,7 @@ enum direction {
 static gunichar vte_terminal_accessible_get_character_at_offset(AtkText *text,
 								gint offset);
 
+/* Create snapshot private data. */
 static VteTerminalAccessiblePrivate *
 vte_terminal_accessible_new_private_data(void)
 {
@@ -73,7 +77,86 @@ vte_terminal_accessible_new_private_data(void)
 	return priv;
 }
 
-/* "Oh yeah, that's selected.  Sure."  */
+/* Free snapshot private data. */
+static void
+vte_terminal_accessible_free_private_data(VteTerminalAccessiblePrivate *priv)
+{
+	g_return_if_fail(priv != NULL);
+	if (priv->snapshot_text != NULL) {
+		g_string_free(priv->snapshot_text, TRUE);
+		priv->snapshot_text = NULL;
+	}
+	if (priv->snapshot_characters != NULL) {
+		g_array_free(priv->snapshot_characters, TRUE);
+		priv->snapshot_characters = NULL;
+	}
+	if (priv->snapshot_attributes != NULL) {
+		g_array_free(priv->snapshot_attributes, TRUE);
+		priv->snapshot_attributes = NULL;
+	}
+	if (priv->snapshot_linebreaks != NULL) {
+		g_array_free(priv->snapshot_linebreaks, TRUE);
+		priv->snapshot_linebreaks = NULL;
+	}
+	g_free(priv);
+}
+
+static gint
+offset_from_xy (VteTerminalAccessiblePrivate *priv, 
+		gint x, gint y)
+{
+	gint offset;
+	gint linebreak;
+	gint next_linebreak;
+
+	if (y >= priv->snapshot_linebreaks->len)
+		y = priv->snapshot_linebreaks->len -1;
+
+	linebreak = g_array_index (priv->snapshot_linebreaks, int, y);
+	if (y +1 == priv->snapshot_linebreaks->len)
+		next_linebreak = priv->snapshot_characters->len;
+	else 
+		next_linebreak = g_array_index (priv->snapshot_linebreaks, int, y + 1);
+	
+	offset = linebreak + x;
+	if (offset >= next_linebreak)
+		offset = next_linebreak -1;
+	return offset;
+}
+
+static void
+xy_from_offset (VteTerminalAccessiblePrivate *priv, 
+		gint offset, gint *x, gint *y)
+{
+	gint i;
+	gint linebreak;
+	gint cur_x, cur_y;
+	gint cur_offset;
+
+	cur_x = -1;
+	cur_y = -1;
+	for (i = 0; i < priv->snapshot_linebreaks->len; i++) {
+		linebreak = g_array_index (priv->snapshot_linebreaks, int, i);
+		if (offset < linebreak) {
+			cur_x = offset - cur_offset;
+			cur_y = i - 1;	
+			break;
+			
+		}  else {
+			cur_offset = linebreak;
+		}
+	}
+	if (i == priv->snapshot_linebreaks->len) {
+		if (offset < priv->snapshot_characters->len) {
+			cur_x = offset - cur_offset;
+			cur_y = i - 1;	
+		}
+	}
+	*x = cur_x;
+	*y = cur_y;	
+}
+
+/* "Oh yeah, that's selected.  Sure." callback. */
 static gboolean
 all_selected(VteTerminal *terminal, glong column, glong row, gpointer data)
 {
@@ -173,7 +256,7 @@ vte_terminal_accessible_update_private_data_if_needed(AtkObject *text,
 
 	g_return_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text));
 
-	/* Retrieve the private data structure. */
+	/* Retrieve the private data structure.  It must already exist. */
 	priv = g_object_get_data(G_OBJECT(text),
 				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
 	g_return_if_fail(priv != NULL);
@@ -205,9 +288,8 @@ vte_terminal_accessible_update_private_data_if_needed(AtkObject *text,
 		return;
 	}
 
-	terminal = VTE_TERMINAL((GTK_ACCESSIBLE(text))->widget);
-
 	/* Re-read the contents of the widget if the contents have changed. */
+	terminal = VTE_TERMINAL((GTK_ACCESSIBLE(text))->widget);
 	if (priv->snapshot_contents_invalid) {
 		/* Free the outdated snapshot data, unless the caller
 		 * wants it. */
@@ -363,30 +445,6 @@ vte_terminal_accessible_update_private_data_if_needed(AtkObject *text,
 #endif
 }
 
-/* Free snapshot private data. */
-static void
-vte_terminal_accessible_free_private_data(VteTerminalAccessiblePrivate *priv)
-{
-	g_return_if_fail(priv != NULL);
-	if (priv->snapshot_text != NULL) {
-		g_string_free(priv->snapshot_text, TRUE);
-		priv->snapshot_text = NULL;
-	}
-	if (priv->snapshot_characters != NULL) {
-		g_array_free(priv->snapshot_characters, TRUE);
-		priv->snapshot_characters = NULL;
-	}
-	if (priv->snapshot_attributes != NULL) {
-		g_array_free(priv->snapshot_attributes, TRUE);
-		priv->snapshot_attributes = NULL;
-	}
-	if (priv->snapshot_linebreaks != NULL) {
-		g_array_free(priv->snapshot_linebreaks, TRUE);
-		priv->snapshot_linebreaks = NULL;
-	}
-	g_free(priv);
-}
-
 /* A signal handler to catch "text-inserted/deleted/modified" signals. */
 static void
 vte_terminal_accessible_text_modified(VteTerminal *terminal, gpointer data)
@@ -607,7 +665,7 @@ vte_terminal_accessible_invalidate_cursor(VteTerminal *terminal, gpointer data)
 							      NULL, NULL);
 }
 
-/* Handle title changes by resetting the parent object. */
+/* Handle title changes by resetting the description. */
 static void
 vte_terminal_accessible_title_changed(VteTerminal *terminal, gpointer data)
 {
@@ -624,6 +682,8 @@ vte_terminal_accessible_focus_in(VteTerminal *terminal, GdkEventFocus *event,
 	g_return_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(data));
 	g_return_if_fail(VTE_IS_TERMINAL(terminal));
 	g_signal_emit_by_name(data, "focus-event", TRUE);
+	atk_object_notify_state_change(ATK_OBJECT(data),
+				       ATK_STATE_FOCUSED, TRUE);
 }
 
 /* Reflect focus-out events. */
@@ -634,6 +694,41 @@ vte_terminal_accessible_focus_out(VteTerminal *terminal, GdkEventFocus *event,
 	g_return_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(data));
 	g_return_if_fail(VTE_IS_TERMINAL(terminal));
 	g_signal_emit_by_name(data, "focus-event", FALSE);
+	atk_object_notify_state_change(ATK_OBJECT(data),
+				       ATK_STATE_FOCUSED, FALSE);
+}
+
+/* Reflect visibility-notify events. */
+static void
+vte_terminal_accessible_visibility_notify(VteTerminal *terminal,
+					  GdkEventVisibility *event,
+					  gpointer data)
+{
+	GtkWidget *widget;
+	gboolean visible;
+	g_return_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(data));
+	g_return_if_fail(VTE_IS_TERMINAL(terminal));
+	visible = event->state != GDK_VISIBILITY_FULLY_OBSCURED;
+	/* The VISIBLE state indicates that this widget is "visible". */
+	atk_object_notify_state_change(ATK_OBJECT(data),
+				       ATK_STATE_VISIBLE,
+				       visible);
+	widget = GTK_WIDGET(terminal);
+	while (visible) {
+		if (gtk_widget_get_toplevel(widget) == widget) {
+			break;
+		}
+		if (widget == NULL) {
+			break;
+		}
+		visible = visible && (GTK_WIDGET_VISIBLE(widget));
+		widget = gtk_widget_get_parent(widget);
+	}
+	/* The SHOWING state indicates that this widget, and all of its
+	 * parents up to the toplevel, are "visible". */
+	atk_object_notify_state_change(ATK_OBJECT(data),
+				       ATK_STATE_SHOWING,
+				       visible);
 }
 
 /**
@@ -660,6 +755,7 @@ vte_terminal_accessible_new(VteTerminal *terminal)
 	atk_object_initialize(ATK_OBJECT(access), G_OBJECT(terminal));
 
 	access->widget = GTK_WIDGET(terminal);
+	_vte_terminal_accessible_ref(terminal);
 	g_object_add_weak_pointer(G_OBJECT(terminal),
 				  (gpointer*)&access->widget);
 
@@ -682,19 +778,24 @@ vte_terminal_accessible_new(VteTerminal *terminal)
 	g_signal_connect(G_OBJECT(terminal), "cursor-moved",
 			 GTK_SIGNAL_FUNC(vte_terminal_accessible_invalidate_cursor),
 			 object);
-        g_signal_connect(G_OBJECT(terminal), "window-title-changed",
+	g_signal_connect(G_OBJECT(terminal), "window-title-changed",
 			 GTK_SIGNAL_FUNC(vte_terminal_accessible_title_changed),
 			 access);
-        g_signal_connect(G_OBJECT(terminal), "focus-in-event",
+	g_signal_connect(G_OBJECT(terminal), "focus-in-event",
 			 GTK_SIGNAL_FUNC(vte_terminal_accessible_focus_in),
 			 access);
-        g_signal_connect(G_OBJECT(terminal), "focus-out-event",
+	g_signal_connect(G_OBJECT(terminal), "focus-out-event",
 			 GTK_SIGNAL_FUNC(vte_terminal_accessible_focus_out),
 			 access);
+	g_signal_connect(G_OBJECT(terminal), "visibility-notify-event",
+			 GTK_SIGNAL_FUNC(vte_terminal_accessible_visibility_notify),
+			 access);
 
 	if (GTK_IS_WIDGET((GTK_WIDGET(terminal))->parent)) {
 		parent = gtk_widget_get_accessible((GTK_WIDGET(terminal))->parent);
-		atk_object_set_parent(ATK_OBJECT(access), parent);
+		if (ATK_IS_OBJECT(parent)) {
+			atk_object_set_parent(ATK_OBJECT(access), parent);
+		}
 	}
 
 	atk_object_set_name(ATK_OBJECT(access), "Terminal");
@@ -702,15 +803,23 @@ vte_terminal_accessible_new(VteTerminal *terminal)
 				   terminal->window_title ?
 				   terminal->window_title :
 				   "");
-	
+
+	atk_object_notify_state_change(ATK_OBJECT(access),
+				       ATK_STATE_FOCUSABLE, TRUE);
+	atk_object_notify_state_change(ATK_OBJECT(access),
+				       ATK_STATE_EXPANDABLE, FALSE);
+	atk_object_notify_state_change(ATK_OBJECT(access),
+				       ATK_STATE_RESIZABLE, TRUE);
+
 	return ATK_OBJECT(access);
 }
 
 static void
 vte_terminal_accessible_finalize(GObject *object)
 {
+	VteTerminalAccessiblePrivate *priv;
 	GtkAccessible *accessible = NULL;
-        GObjectClass *gobject_class; 
+	GObjectClass *gobject_class;
 
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_MISC)) {
@@ -731,19 +840,19 @@ vte_terminal_accessible_finalize(GObject *object)
 						     G_SIGNAL_MATCH_FUNC |
 						     G_SIGNAL_MATCH_DATA,
 						     0, 0, NULL,
-						     vte_terminal_accessible_text_modified,
+						     (gpointer)vte_terminal_accessible_text_modified,
 						     object);
 		g_signal_handlers_disconnect_matched(G_OBJECT(accessible->widget),
 						     G_SIGNAL_MATCH_FUNC |
 						     G_SIGNAL_MATCH_DATA,
 						     0, 0, NULL,
-						     vte_terminal_accessible_text_scrolled,
+						     (gpointer)vte_terminal_accessible_text_scrolled,
 						     object);
 		g_signal_handlers_disconnect_matched(G_OBJECT(accessible->widget),
 						     G_SIGNAL_MATCH_FUNC |
 						     G_SIGNAL_MATCH_DATA,
 						     0, 0, NULL,
-						     vte_terminal_accessible_invalidate_cursor,
+						     (gpointer)vte_terminal_accessible_invalidate_cursor,
 						     object);
 		g_signal_handlers_disconnect_matched(G_OBJECT(accessible->widget),
 						     G_SIGNAL_MATCH_FUNC |
@@ -763,6 +872,20 @@ vte_terminal_accessible_finalize(GObject *object)
 						     0, 0, NULL,
 						     (gpointer)vte_terminal_accessible_focus_out,
 						     object);
+		g_signal_handlers_disconnect_matched(G_OBJECT(accessible->widget),
+						     G_SIGNAL_MATCH_FUNC |
+						     G_SIGNAL_MATCH_DATA,
+						     0, 0, NULL,
+						     (gpointer)vte_terminal_accessible_visibility_notify,
+						     object);
+	}
+	priv = g_object_get_data(object,
+				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
+	if (priv != NULL) {
+		vte_terminal_accessible_free_private_data(priv);
+		g_object_set_data(object,
+				  VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA,
+				  NULL);
 	}
 	if (gobject_class->finalize != NULL) {
 		gobject_class->finalize(object);
@@ -1087,24 +1210,90 @@ vte_terminal_accessible_get_caret_offset(AtkText *text)
 	return priv->snapshot_caret;
 }
 
+static AtkAttributeSet *
+get_attribute_set (struct _VteCharAttributes attr)
+{
+	AtkAttributeSet *set = NULL;
+	AtkAttribute *at;
+
+	if (attr.underline) {
+		at = g_new (AtkAttribute, 1);
+		at->name = g_strdup ("underline");
+		at->value = g_strdup ("true");
+		set = g_slist_append (set, at);
+	}
+	if (attr.strikethrough) {
+		at = g_new (AtkAttribute, 1);
+		at->name = g_strdup ("strikethrough");
+		at->value = g_strdup ("true");
+		set = g_slist_append (set, at);
+	}
+	at = g_new (AtkAttribute, 1);
+	at->name = g_strdup ("fg-color");
+	at->value = g_strdup_printf ("%u,%u,%u", 
+				     attr.fore.red, attr.fore.green, attr.fore.blue);
+	set = g_slist_append (set, at);
+
+	at = g_new (AtkAttribute, 1);
+	at->name = g_strdup ("bg-color");
+	at->value = g_strdup_printf ("%u,%u,%u", 
+				     attr.back.red, attr.back.green, attr.back.blue);
+	set = g_slist_append (set, at);
+
+	return set;
+}
+	
 static AtkAttributeSet *
 vte_terminal_accessible_get_run_attributes(AtkText *text, gint offset,
 					   gint *start_offset, gint *end_offset)
 {
-	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), NULL);
+	VteTerminalAccessiblePrivate *priv;
+	gint i;
+	struct _VteCharAttributes cur_attr;
+	struct _VteCharAttributes attr;
+
 	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
 							      NULL, NULL);
-	/* FIXME */
-	return NULL;
+
+	priv = g_object_get_data(G_OBJECT(text),
+				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
+
+	attr = g_array_index (priv->snapshot_attributes,
+			      struct _VteCharAttributes,
+			      offset);
+	*start_offset = 0;
+	for (i = offset - 1; i >= 0; i--) {
+		cur_attr = g_array_index (priv->snapshot_attributes,
+				      struct _VteCharAttributes,
+				      i);
+		if (!gdk_color_equal (&cur_attr.fore, &attr.fore) ||
+		    !gdk_color_equal (&cur_attr.back, &attr.back) ||
+		    cur_attr.underline != attr.underline ||
+		    cur_attr.strikethrough != attr.strikethrough) {
+			*start_offset = i + 1;	
+			break;
+		}
+	}
+	*end_offset = priv->snapshot_attributes->len - 1;
+	for (i = offset + 1; i < priv->snapshot_attributes->len; i++) {
+		cur_attr = g_array_index (priv->snapshot_attributes,
+				      struct _VteCharAttributes,
+				      i);
+		if (!gdk_color_equal (&cur_attr.fore, &attr.fore) ||
+		    !gdk_color_equal (&cur_attr.back, &attr.back) ||
+		    cur_attr.underline != attr.underline ||
+		    cur_attr.strikethrough != attr.strikethrough) {
+			*end_offset = i - 1;
+			break;
+		}
+	}
+	
+	return get_attribute_set (attr);
 }
 
 static AtkAttributeSet *
 vte_terminal_accessible_get_default_attributes(AtkText *text)
 {
-	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), NULL);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
-							      NULL, NULL);
-	/* FIXME */
 	return NULL;
 }
 
@@ -1114,10 +1303,29 @@ vte_terminal_accessible_get_character_extents(AtkText *text, gint offset,
 					      gint *width, gint *height,
 					      AtkCoordType coords)
 {
+	VteTerminalAccessiblePrivate *priv;
+	VteTerminal *terminal;
+	glong char_width, char_height;
+	gint base_x, base_y;
+
 	g_return_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text));
+
 	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
 							      NULL, NULL);
-	/* FIXME */
+	priv = g_object_get_data(G_OBJECT(text),
+				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
+	terminal = VTE_TERMINAL (GTK_ACCESSIBLE (text)->widget);
+
+	atk_component_get_position (ATK_COMPONENT (text), &base_x, &base_y, coords);
+	xy_from_offset (priv, offset, x, y);
+	char_width = vte_terminal_get_char_width (terminal);
+	char_height = vte_terminal_get_char_height (terminal);
+	*x *= char_width;
+	*y *= char_height;
+	*width = char_width;
+	*height = char_height;
+	*x += base_x;
+	*y += base_y;
 }
 
 static gint
@@ -1139,11 +1347,27 @@ vte_terminal_accessible_get_offset_at_point(AtkText *text,
 					    gint x, gint y,
 					    AtkCoordType coords)
 {
-	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), 0);
+	VteTerminalAccessiblePrivate *priv;
+	VteTerminal *terminal;
+	glong char_width, char_height;
+	gint base_x, base_y;
+
+	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), -1);
+
 	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
 							      NULL, NULL);
-	/* FIXME */
-	return 0;
+	priv = g_object_get_data(G_OBJECT(text),
+				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
+	terminal = VTE_TERMINAL (GTK_ACCESSIBLE (text)->widget);
+
+	atk_component_get_position (ATK_COMPONENT (text), &base_x, &base_y, coords);
+	char_width = vte_terminal_get_char_width (terminal);
+	char_height = vte_terminal_get_char_height (terminal);
+	x -= base_x;
+	y -= base_y;
+	x /= char_width;
+	y /= char_height;
+	return offset_from_xy (priv, x, y);
 }
 
 static gint
@@ -1216,6 +1440,12 @@ vte_terminal_accessible_text_init(gpointer iface, gpointer data)
 	AtkTextIface *text;
 	g_return_if_fail(G_TYPE_FROM_INTERFACE(iface) == ATK_TYPE_TEXT);
 	text = iface;
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_MISC)) {
+		fprintf(stderr, "Initializing accessible peer's "
+			"AtkText interface.\n");
+	}
+#endif
 	text->get_text = vte_terminal_accessible_get_text;
 	text->get_text_after_offset = vte_terminal_accessible_get_text_after_offset;
 	text->get_text_at_offset = vte_terminal_accessible_get_text_at_offset;
@@ -1235,42 +1465,213 @@ vte_terminal_accessible_text_init(gpointer iface, gpointer data)
 	text->set_caret_offset = vte_terminal_accessible_set_caret_offset;
 }
 
-static void
-vte_terminal_accessible_text_finalize(gpointer iface, gpointer data)
+static AtkLayer
+vte_terminal_accessible_get_layer(AtkComponent *component)
 {
-	GtkAccessibleClass *accessible_class;
-	VteTerminalAccessiblePrivate *priv;
-	accessible_class = g_type_class_peek(GTK_TYPE_ACCESSIBLE); 
+	return ATK_LAYER_WIDGET;
+}
+
+static gint
+vte_terminal_accessible_get_mdi_zorder(AtkComponent *component)
+{
+	return G_MININT;
+}
+
+static gboolean
+vte_terminal_accessible_contains(AtkComponent *component,
+				 gint x, gint y,
+				 AtkCoordType coord_type)
+{
+	gint ex, ey, ewidth, eheight;
+	atk_component_get_extents(component, &ex, &ey, &ewidth, &eheight,
+				  coord_type);
+	return ((x >= ex) &&
+		(x < ex + ewidth) &&
+		(y >= ey) &&
+		(y < ey + eheight));
+}
+
+static void
+vte_terminal_accessible_get_extents(AtkComponent *component,
+				    gint *x, gint *y,
+				    gint *width, gint *height,
+				    AtkCoordType coord_type)
+{
+	atk_component_get_position(component, x, y, coord_type);
+	atk_component_get_size(component, width, height);
+}
+
+static void
+vte_terminal_accessible_get_position(AtkComponent *component,
+				     gint *x, gint *y,
+				     AtkCoordType coord_type)
+{
+	GtkWidget *widget;
+	*x = 0;
+	*y = 0;
+	widget = (GTK_ACCESSIBLE(component))->widget;
+	if (!GTK_WIDGET_REALIZED(widget)) {
+		return;
+	}
+	switch (coord_type) {
+	case ATK_XY_SCREEN:
+		gdk_window_get_origin(widget->window, x, y);
+		break;
+	case ATK_XY_WINDOW:
+		gdk_window_get_position(widget->window, x, y);
+		break;
+	default:
+		g_assert_not_reached();
+		break;
+	}
+}
+
+static void
+vte_terminal_accessible_get_size(AtkComponent *component,
+				 gint *width, gint *height)
+{
+	GtkWidget *widget;
+	*width = 0;
+	*height = 0;
+	widget = (GTK_ACCESSIBLE(component))->widget;
+	if (!GTK_WIDGET_REALIZED(widget)) {
+		return;
+	}
+	gdk_drawable_get_size(widget->window, width, height);
+}
+
+static gboolean
+vte_terminal_accessible_set_extents(AtkComponent *component,
+				    gint x, gint y,
+				    gint width, gint height,
+				    AtkCoordType coord_type)
+{
+	/* FIXME?  We can change the size, but our position is controlled
+	 * by the parent container. */
+	return FALSE;
+}
+
+static gboolean
+vte_terminal_accessible_set_position(AtkComponent *component,
+				     gint x, gint y,
+				     AtkCoordType coord_type)
+{
+	/* Controlled by the parent container, if there is one. */
+	return FALSE;
+}
+
+static gboolean
+vte_terminal_accessible_set_size(AtkComponent *component,
+				 gint width, gint height)
+{
+	VteTerminal *terminal;
+	gint columns, rows, xpad, ypad;
+	terminal = VTE_TERMINAL((GTK_ACCESSIBLE(component))->widget);
+	vte_terminal_get_padding(terminal, &xpad, &ypad);
+	/* If the size is an exact multiple of the cell size, use that,
+	 * otherwise round down. */
+	if (width % terminal->char_width == 0) {
+		columns = width / terminal->char_width;
+	} else {
+		columns = (width + xpad) / terminal->char_width;
+	}
+	if (height % terminal->char_height == 0) {
+		rows = height / terminal->char_height;
+	} else {
+		rows = (height + xpad) / terminal->char_height;
+	}
+	vte_terminal_set_size(terminal, columns, rows);
+	return (terminal->row_count == rows) &&
+	       (terminal->column_count == columns);
+}
+
+static gboolean
+vte_terminal_accessible_grab_focus(AtkComponent *component)
+{
+	GtkWidget *widget;
+	widget = (GTK_ACCESSIBLE(component))->widget;
+	if (GTK_WIDGET_HAS_FOCUS(widget)) {
+		return TRUE;
+	}
+	gtk_widget_grab_focus(widget);
+	if (GTK_WIDGET_HAS_FOCUS(widget)) {
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static AtkObject *
+vte_terminal_accessible_ref_accessible_at_point(AtkComponent *component,
+						gint x, gint y,
+						AtkCoordType coord_type)
+{
+	/* There are no children. */
+	return NULL;
+}
+
+static guint
+vte_terminal_accessible_add_focus_handler(AtkComponent *component,
+					  AtkFocusHandler handler)
+{
+	guint signal_id;
+	signal_id = g_signal_lookup("focus-event",
+				    VTE_TYPE_TERMINAL_ACCESSIBLE);
+	if (g_signal_handler_find(component,
+				  G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_ID,
+				  signal_id,
+				  0,
+				  NULL,
+				  (gpointer)handler,
+				  NULL) != 0) {
+		return 0;
+	}
+	return g_signal_connect(component, "focus-event",
+				G_CALLBACK(handler), NULL);
+}
+
+static void
+vte_terminal_accessible_remove_focus_handler(AtkComponent *component,
+					     guint handler_id)
+{
+	g_return_if_fail(g_signal_handler_is_connected(component, handler_id));
+	g_signal_handler_disconnect(component, handler_id);
+}
+
+static void
+vte_terminal_accessible_component_init(gpointer iface, gpointer data)
+{
+	AtkComponentIface *component;
+	g_return_if_fail(G_TYPE_FROM_INTERFACE(iface) == ATK_TYPE_COMPONENT);
+	component = iface;
 
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_MISC)) {
-		fprintf(stderr, "Finalizing accessible peer AtkText interface.\n");
+		fprintf(stderr, "Initializing accessible peer's "
+			"AtkComponent interface.\n");
 	}
 #endif
-
-	/* Free the private data. */
-	priv = g_object_get_data(G_OBJECT(iface),
-				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
-	if (priv) {
-		vte_terminal_accessible_free_private_data(priv);
-		g_object_set_data(G_OBJECT(iface),
-				  VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA,
-				  NULL);
-	}
-
-	if ((G_OBJECT_CLASS(accessible_class))->finalize) {
-		(G_OBJECT_CLASS(accessible_class))->finalize(iface);
-	}
+	/* Set our virtual functions. */
+	component->add_focus_handler = vte_terminal_accessible_add_focus_handler;
+	component->contains = vte_terminal_accessible_contains;
+	component->ref_accessible_at_point = vte_terminal_accessible_ref_accessible_at_point;
+	component->get_extents = vte_terminal_accessible_get_extents;
+	component->get_position = vte_terminal_accessible_get_position;
+	component->get_size = vte_terminal_accessible_get_size;
+	component->grab_focus = vte_terminal_accessible_grab_focus;
+	component->remove_focus_handler = vte_terminal_accessible_remove_focus_handler;
+	component->set_extents = vte_terminal_accessible_set_extents;
+	component->set_position = vte_terminal_accessible_set_position;
+	component->set_size = vte_terminal_accessible_set_size;
+	component->get_layer = vte_terminal_accessible_get_layer;
+	component->get_mdi_zorder = vte_terminal_accessible_get_mdi_zorder;
 }
 
 static void
 vte_terminal_accessible_class_init(gpointer *klass)
 {
-        GObjectClass *gobject_class; 
+	GObjectClass *gobject_class;
 
-	bindtextdomain(PACKAGE, LOCALEDIR);
-
-        gobject_class = G_OBJECT_CLASS(klass); 
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	/* Override the finalize method. */
 	gobject_class->finalize = vte_terminal_accessible_finalize;
@@ -1279,23 +1680,32 @@ vte_terminal_accessible_class_init(gpointer *klass)
 static void
 vte_terminal_accessible_init(gpointer *instance, gpointer *klass)
 {
-	/* Mark the role this object plays. The rest of the work is handled
-	 * by the AtkText interface the object class exports. */
+	/* Mark the role this object plays. */
 	g_return_if_fail(ATK_IS_OBJECT(instance));
 	atk_object_set_role(ATK_OBJECT(instance), ATK_ROLE_TERMINAL);
 }
 
-GtkType
+GType
 vte_terminal_accessible_get_type(void)
 {
-	static GtkType terminal_accessible_type = 0;
+	AtkRegistry *registry;
+	AtkObjectFactory *factory;
+	GType parent_type, parent_accessible_type;
+	GTypeQuery type_info;
+
+	static GType terminal_accessible_type = 0;
 	static GInterfaceInfo text = {
 		vte_terminal_accessible_text_init,
-		vte_terminal_accessible_text_finalize,
+		NULL,
 		NULL,
 	};
-	static const GTypeInfo terminal_accessible_info = {
-		sizeof(VteTerminalAccessibleClass),
+	static GInterfaceInfo component = {
+		vte_terminal_accessible_component_init,
+		NULL,
+		NULL,
+	};
+	static GTypeInfo terminal_accessible_info = {
+		0,
 		(GBaseInitFunc)NULL,
 		(GBaseFinalizeFunc)NULL,
 
@@ -1311,8 +1721,30 @@ vte_terminal_accessible_get_type(void)
 	};
 
 	if (terminal_accessible_type == 0) {
+		/* Find the Atk object used for the parent (GtkWidget) type. */
+		parent_type = g_type_parent(VTE_TYPE_TERMINAL);
+		factory = atk_registry_get_factory(atk_get_default_registry(),
+						   parent_type);
+		parent_accessible_type = atk_object_factory_get_accessible_type(factory);
+		if (!g_type_is_a(parent_accessible_type, GTK_TYPE_ACCESSIBLE)) {
+#ifdef VTE_DEBUG
+			g_warning("Accessibility (%s) is not derived from "
+				  "%s (GTK_MODULES=gail not set?), "
+				  "deriving from %s instead.\n",
+				  g_type_name(parent_accessible_type),
+				  g_type_name(GTK_TYPE_ACCESSIBLE),
+				  g_type_name(GTK_TYPE_ACCESSIBLE));
+#endif
+			/* Fudge it. */
+			parent_accessible_type = GTK_TYPE_ACCESSIBLE;
+		}
+
+		/* Find the size of the parent type's objects. */
+		g_type_query(parent_accessible_type, &type_info);
+		terminal_accessible_info.class_size = type_info.class_size;
+		terminal_accessible_info.instance_size = type_info.instance_size;
 		/* Register the class with the GObject type system. */
-		terminal_accessible_type = g_type_register_static(GTK_TYPE_ACCESSIBLE,
+		terminal_accessible_type = g_type_register_static(parent_accessible_type,
 								  "VteTerminalAccessible",
 								  &terminal_accessible_info,
 								  0);
@@ -1321,7 +1753,85 @@ vte_terminal_accessible_get_type(void)
 		g_type_add_interface_static(terminal_accessible_type,
 					    ATK_TYPE_TEXT,
 					    &text);
+		/* Add a component interface to this object class. */
+		g_type_add_interface_static(terminal_accessible_type,
+					    ATK_TYPE_COMPONENT,
+					    &component);
+
+		/* Associate the terminal and its peer factory in the
+		 * Atk type registry. */
+		registry = atk_get_default_registry();
+		atk_registry_set_factory_type(registry,
+					      VTE_TYPE_TERMINAL,
+					      VTE_TYPE_TERMINAL_ACCESSIBLE_FACTORY);
 	}
 
 	return terminal_accessible_type;
 }
+
+/* Create an accessible peer for the object. */
+static AtkObject *
+vte_terminal_accessible_factory_create_accessible(GObject *obj)
+{
+	GtkAccessible *accessible;
+	VteTerminal *terminal;
+
+	g_return_val_if_fail(VTE_IS_TERMINAL(obj), NULL);
+
+	terminal = VTE_TERMINAL(obj);
+	accessible = GTK_ACCESSIBLE(vte_terminal_accessible_new(terminal));
+	g_return_val_if_fail(accessible != NULL, NULL);
+
+	return ATK_OBJECT(accessible);
+}
+
+static void
+vte_terminal_accessible_factory_class_init(VteTerminalAccessibleFactoryClass *klass)
+{
+	AtkObjectFactoryClass *class = ATK_OBJECT_FACTORY_CLASS(klass);
+	/* Override the one method we care about. */
+	class->create_accessible = vte_terminal_accessible_factory_create_accessible;
+}
+
+AtkObjectFactory *
+vte_terminal_accessible_factory_new(void)
+{
+	GObject *factory;
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_MISC)) {
+		fprintf(stderr, "Creating a new "
+			"VteTerminalAccessibleFactory.\n");
+	}
+#endif
+	factory = g_object_new(VTE_TYPE_TERMINAL_ACCESSIBLE_FACTORY, NULL);
+	g_return_val_if_fail(factory != NULL, NULL);
+	return ATK_OBJECT_FACTORY(factory);
+}
+
+GType
+vte_terminal_accessible_factory_get_type(void)
+{
+	static GType terminal_accessible_factory_type = 0;
+	static GTypeInfo terminal_accessible_factory_type_info = {
+		sizeof(VteTerminalAccessibleFactoryClass),
+		(GBaseInitFunc)NULL,
+		(GBaseFinalizeFunc)NULL,
+
+		(GClassInitFunc)vte_terminal_accessible_factory_class_init,
+		(GClassFinalizeFunc)NULL,
+		(gconstpointer)NULL,
+
+		sizeof(VteTerminalAccessibleFactory),
+		0,
+		(GInstanceInitFunc)NULL,
+
+		(GTypeValueTable*)NULL,
+	};
+	if (terminal_accessible_factory_type == 0) {
+		terminal_accessible_factory_type = g_type_register_static(ATK_TYPE_OBJECT_FACTORY,
+									  "VteTerminalAccessibleFactory",
+									  &terminal_accessible_factory_type_info,
+									  0);
+	}
+	return terminal_accessible_factory_type;
+}
