@@ -18,7 +18,7 @@
 
 /* The interfaces in this file are subject to change at any time. */
 
-#ident "$Id: vtedraw.c,v 1.11 2003/05/06 07:38:39 nalin Exp $"
+#ident "$Id: vtedraw.c,v 1.14 2003/05/28 21:53:59 nalin Exp $"
 
 #include "../config.h"
 
@@ -44,18 +44,18 @@ struct _vte_draw_impl
 #endif
 #endif
 	&_vte_draw_ft2,
-	&_vte_draw_pango,
-#ifndef X_DISPLAY_MISSING
-#ifdef HAVE_PANGOX
-	&_vte_draw_pango_x,
-#endif
-#endif
 #if GTK_CHECK_VERSION(2,2,0)
 #ifndef X_DISPLAY_MISSING
 #ifdef HAVE_GL
 	&_vte_draw_gl,
 #endif
 #endif
+#endif
+	&_vte_draw_pango,
+#ifndef X_DISPLAY_MISSING
+#ifdef HAVE_PANGOX
+	&_vte_draw_pango_x,
+#endif
 #endif
 };
 
@@ -126,17 +126,28 @@ _vte_draw_get_visual(struct _vte_draw *draw)
 }
 
 GdkColormap *
-_vte_draw_get_colormap(struct _vte_draw *draw)
+_vte_draw_get_colormap(struct _vte_draw *draw, gboolean maybe_use_default)
 {
 	GdkColormap *colormap;
+#if GTK_CHECK_VERSION(2,2,0)
+	GdkScreen *screen;
+#endif
 	g_return_val_if_fail(draw->impl != NULL, NULL);
 	g_return_val_if_fail(draw->impl->get_colormap != NULL, NULL);
 	colormap = draw->impl->get_colormap(draw);
 	if (colormap) {
 		return colormap;
 	}
+	if (!maybe_use_default) {
+		return NULL;
+	}
 #if GTK_CHECK_VERSION(2,2,0)
-	colormap = gdk_screen_get_default_colormap(gdk_screen_get_default());
+	if (gtk_widget_has_screen(draw->widget)) {
+		screen = gtk_widget_get_screen(draw->widget);
+	} else {
+		screen = gdk_display_get_default_screen(gtk_widget_get_display(draw->widget));
+	}
+	colormap = gdk_screen_get_default_colormap(screen);
 #else
 	colormap = gdk_colormap_get_system();
 #endif
@@ -186,6 +197,14 @@ _vte_draw_set_background_image(struct _vte_draw *draw,
 	draw->impl->set_background_image(draw, type, pixbuf, filename,
 					 color, saturation);
 }
+
+gboolean
+_vte_draw_requires_repaint(struct _vte_draw *draw)
+{
+	g_return_val_if_fail(draw->impl != NULL, TRUE);
+	return draw->impl->requires_repaint;
+}
+
 void
 _vte_draw_clear(struct _vte_draw *draw, gint x, gint y, gint width, gint height)
 {
@@ -246,6 +265,20 @@ _vte_draw_text(struct _vte_draw *draw,
 	draw->impl->draw_text(draw, requests, n_requests, color, alpha);
 }
 
+gboolean
+_vte_draw_char(struct _vte_draw *draw,
+	       struct _vte_draw_text_request *request,
+	       GdkColor *color, guchar alpha)
+{
+	g_return_val_if_fail(draw->started == TRUE, FALSE);
+	g_return_val_if_fail(draw->impl != NULL, FALSE);
+	if (draw->impl->draw_char == NULL) {
+		draw->impl->draw_text(draw, request, 1, color, alpha);
+		return TRUE;
+	}
+	return draw->impl->draw_char(draw, request, color, alpha);
+}
+
 void
 _vte_draw_fill_rectangle(struct _vte_draw *draw,
 			 gint x, gint y, gint width, gint height,
