@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vteskel.c,v 1.7 2003/05/13 04:27:39 nalin Exp $"
+#ident "$Id: vteskel.c,v 1.8 2003/05/28 21:53:59 nalin Exp $"
 
 #include "../config.h"
 
@@ -176,6 +176,16 @@ _vte_skel_draw_text(struct _vte_draw *draw,
 	data = (struct _vte_skel_data*) draw->impl_data;
 }
 
+static gboolean
+_vte_skel_draw_char(struct _vte_draw *draw,
+		    struct _vte_draw_text_request *request,
+		    GdkColor *color, guchar alpha)
+{
+	struct _vte_skel_data *data;
+	data = (struct _vte_skel_data*) draw->impl_data;
+	return FALSE;
+}
+
 static void
 _vte_skel_draw_rectangle(struct _vte_draw *draw,
 			 gint x, gint y, gint width, gint height,
@@ -222,6 +232,7 @@ struct _vte_draw_impl _vte_draw_skel = {
 	_vte_skel_get_text_ascent,
 	_vte_skel_get_using_fontconfig,
 	_vte_skel_draw_text,
+	_vte_skel_draw_char,
 	_vte_skel_draw_rectangle,
 	_vte_skel_fill_rectangle,
 	_vte_skel_set_scroll,
